#!/bin/bash
# Licensed Materials - Property of IBM
#
# 09N6996
#
# (C) Copyright IBM Corp. 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#############################################################

dflag=0
vflags=""
RC=0

CURRENT_CFG=/var/ct/cfg/current_cluster
DEFAULT_CFG=/var/ct/cfg/default_cluster
CLUSTERS=/var/ct/cfg/clusters

PARSE_SRC_STATE="-e1d;s/.* \([a-zA-Z0-9]*\)$/\1/"

debug () {
    if [ $dflag -eq 1 ]; then
        TS=$(date)
        echo "$TS $*"
    fi

    return 0
}

do_tsa_rsct_update () {
    local IWflag=""

    debug "start TSA/RSCT update"

    # check cluster mode; if not IW, then make it IW for update
    #
    # N.B. if for some odd reason lsclcfg fails, then do the updates.
    # However, if lsclcfg fails, then the cluster is in a strange
    # state and the updates could also fail.

    clustermode=$(lsclcfg -x 2> /dev/null | cut -d" " -f1)
    debug "current cluster mode is \"$clustermode\""

    if [ -n "$clustermode" -a "$clustermode" != "IW" ]
    then
        # get the cluster ID for IW

        while read CID CNAME
        do
            if [ "$CNAME" = "IW" ]
            then
                IWflag=1
                break
            fi
        done < $CLUSTERS

        if [ -n "$IWflag" ]
        then
            debug "temporarily set cluster mode to IW"

            mv -f $CURRENT_CFG ${CURRENT_CFG}.save
            mv -f $DEFAULT_CFG ${DEFAULT_CFG}.save

            echo $CID >  $CURRENT_CFG
            echo $CID >  $DEFAULT_CFG
        else
            debug "Could not find cluster ID for IW"
        fi
    fi

    export INUCLIENTS=1        # flag to RSCT to not start daemons

    # now do the actual updates, first RSCT, then TSA

    rpm -U${vflags} $RSCT_RPMS 2>&1
    rpm -U${vflags} $TSA_RPMS  2>&1

    if [ -n "$IWflag" ]
    then
        debug "set cluster mode back to $clustermode"

        # restore cluster cfg files

        mv -f ${CURRENT_CFG}.save $CURRENT_CFG
        mv -f ${DEFAULT_CFG}.save $DEFAULT_CFG
    fi

    debug "TSA/RSCT update is complete"
    return 0
}

print_usage () {
    echo "usage: [-d ] [ -v ] path_to_rpms"
}

# main

while getopts ":dv" opt; do
    case $opt in
        d  ) dflag=1 ;;
        v  ) vflags="vvh" ;;
        \? ) print_usage; exit 1 ;;
    esac
done

shift $(( OPTIND -1 ))
RPMDIR=$1

if [ -z "$RPMDIR" -o ! -d "$RPMDIR" ]
then
    print_usage
    exit 1
fi

cd $RPMDIR
TSA_RPMS=$(ls sam* 2> /dev/null)
if [ -z "$TSA_RPMS" ]
then
    debug "No TSA RPMs found"
fi

RSCT_RPMS=$(ls rsct* 2> /dev/null)
if [ -z "$RSCT_RPMS" ]
then
    debug "No RSCT RPMs found"
fi

if [ -n "$TSA_RPMS" -o -n "$RSCT_RPMS" ]
then
    debug "Found RPMs to update => $TSA_RPMS"
    debug "Found RPMs to update => $RSCT_RPMS"

    SRCPID=""
    SRCMSTR=$(ps -e | fgrep srcmstr)
    if [ -z "$SRCMSTR" ]
    then
        /sbin/srcmstr &
        SRCPID=$!

        debug "srcmstr has been started"
        sleep 3     # give time for the srcmstr daemon to start
    fi

    # check that 1) srcmstr is running and 2) RSCT is installed
    LC_ALL=C lssrc -a | fgrep ctrmc >& /dev/null
    RC=$?
    if [ $RC -eq 0 ]
    then
        # check that RSCT is not running (we assume this is a deferred update !!)
        state=$(export LC_ALL=C; lssrc -s ctrmc | sed "$PARSE_SRC_STATE")
        if [ -n "$state" -a "$state" = "inoperative" ]
        then
            do_tsa_rsct_update
        else
            echo "RSCT is active; cannot do update"
            RC=1
        fi
    else
        LC_ALL=C lssrc -a 
        echo "srcmstr not running or RSCT not installed; cannot do update"
    fi

    # kill srcmstr if started above since for normal operation it must be
    # started by the init process

    if [ -n "$SRCPID" ]
    then
        debug "stopping srcmstr"
        kill -9 $SRCPID
    fi
fi

exit $RC
